Attribute VB_Name = "modRubberBand"
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' Author:  Brad Perkins
'' Date:    Nov 1997
'' bradley@gvn.net
'' http://pluto.gvn.net/bradley
''
'' DemoApp Purpose:  RubberBand a rectangular region and return
'' a RECT.  Uses Several API functions to provide smooth, flicker-
'' free drawing.  This code is derived from various Microsoft
'' sample 'C' code.
''
'' Usage granted as long as this Header Remains Intact and credit
'' given where Credit is Due.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''      Modified Feb 4 1998; Brad Perkins
'' Cut out of xCapture and removed Image capure specifics..
'' Changed to return a rect instead of a picture object
'' for users requesting this function
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' CaptureRect(HWND hwnd) As Picture
''
'' This function allows the user to rubber-band a portion of the
'' screen. When the left button is released, the rect that the
'' user selected (in screen coordinates) is converted to a
'' bitmap and then a picture object that is returned
''
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Function CaptureRect(ByVal hWnd As Long, ptOrigin As POINTAPI) As RECT
    
    Dim hDC As Long
    Dim msg As msg
    Dim hPal, hBitmap As Long
    Dim rcClip As RECT
    Dim rcClient As RECT
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' Device Context for screen
    hDC = CreateDCNull("DISPLAY", 0, 0, 0&)
   
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' Convert to screen coordinates
    ClientToScreen hWnd, ptOrigin

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' rcClip is the current rectangle that the user has
    '' selected.  Since user just pressed left button down,
    '' initialize this rect very small
    rcClip.Left = ptOrigin.x
    rcClip.Right = ptOrigin.x
    
    rcClip.Top = ptOrigin.y
    rcClip.Bottom = ptOrigin.y
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' Make sure it is a normal rect
    NormalizeRect rcClip
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' Begin Drawing the rubber-band box
    DrawSelect hDC, rcClip

    ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' Eat mouse messages until a WM_LBUTTONUP is encountered.
    '' Meanwhile continue to draw a rubberbanding rectangle
    '' and display it's dimensions.  I originally had this in
    '' the forms various Mouse Events but it turned out to
    '' be too pokey probably because of the checks that had to
    '' be made on each msg.  This is real smooth, even on my
    '' lowly 166. For you VB purists, (you know who you are)
    '' Tough Cookies!
    Do
        '''''''''''''''''''''''''''''''''''''''''''''''
        '' Wait for any mouse message
        WaitMessage
        
        If (PeekMessage(msg, 0, WM_MOUSEFIRST, _
                WM_MOUSELAST, PM_REMOVE)) Then
                
            ''''''''''''''''''''''''''''''''''''''''''''
            '' Erase the previous rectangle
            DrawSelect hDC, rcClip
            
            ''''''''''''''''''''''''''''''''''''''''''''
            '' Get new coordinates
            rcClip.Left = ptOrigin.x
            rcClip.Top = ptOrigin.y
            rcClip.Right = msg.pt.x
            rcClip.Bottom = msg.pt.y

            ''''''''''''''''''''''''''''''''''''''''''''
            '' See if we swapped ends
            NormalizeRect rcClip
            
            ''''''''''''''''''''''''''''''''''''''''''''
            '' Draw the new Rectangle
            DrawSelect hDC, rcClip

            If (msg.message = WM_LBUTTONUP) Then Exit Do
        
        End If
        
    '''''''''''''''''''''''''''''''''''''''''''''
    '' Next Mouse message
    Loop
        
    '''''''''''''''''''''''''''''''''''''''''''
    '' Erase the last rect
    DrawSelect hDC, rcClip

    '''''''''''''''''''''''''''''''''''''''''''
    '' Clean up DC
    DeleteDC hDC
        
    ''''''''''''''''''''''''''''''''''''''''''''''
    '' Release the capture
    ReleaseCapture

    ''''''''''''''''''''''''''''''''''''''''''''''
    '' Set The Rect
    CaptureRect = rcClip
    
    '''''''''''''''''''''''''''''''''''''''''''''''
    '' Post the ButtonUp message back to the Form
    '' To let us know we are done
    PostMessage hWnd, WM_LBUTTONUP, msg.wParam, msg.lParam

''''''''''''''''''''''''''''''''''''''''''''''
'' End Capture rect
End Function


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' DrawSelect
''
'' Draws the selected clip rectangle with its dimensions
'' to the DC
Public Function DrawSelect(hDC As Long, rcClip As RECT)

    Dim hdcBits As Long     '' DWORD
    Dim x, y As Long
    Dim dx, dy As Long
    Dim hbm As Long         '' HBITMAP
    Dim sExtent As Size     '' SIZE
    Dim sz As String
    Dim psz As Long         '' Size sz

            
    '''''''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' If a rectangular clip region has been selected,
    '' draw it
    If Not IsRectEmpty(rcClip) Then
        
        ''''''''''''''''''''''''''''''''''''''''''''''''''''''
        '' The PatBlt function paints the given rectangle
        '' using the brush that is currently selected into
        '' the specified device context. The brush color and
        '' the surface color(s) are combined by using the
        '' given raster operation.
        ''
        '' BOOL PatBlt(hdc,nXLeft,nYLeft,nWidth,nHeight, _
        ''                 dwRop
        ''
        '' hdc: handle to device context
        '' nXLeft: x-coord. of upper-left corner of rect. to
        ''         be filled
        '' nYLeft: y-coord. of upper-left corner of rect. to
        ''         be filled
        '' nWidth: width of rectangle to be filled
        '' nHeight: height of rectangle to be fille
        '' dwRop: raster operation code
        ''
        '' Parameters:
        '' hDC: Identifies the device context.
        '' nXLeft: Specifies the x-coordinate, in logical
        ''         units, of the upper-left corner of the
        ''         rectangle to be filled.
        '' nYLeft: Specifies the y-coordinate, in logical
        ''         units, of the upper-left corner of the
        ''         rectangle to be filled.
        '' nWidth: Specifies the width, in logical units,
        ''         of the rectangle.
        '' nHeight: Specifies the height, in logical units,
        ''          of the rectangle.
        '' dwRop: Specifies the raster operation code.
        ''        This code may be one of the following
        ''        values:
        ''''''''''''''''''''''''''''''''''''''''''''''''''''''
        PatBlt hDC, rcClip.Left, rcClip.Top, _
            rcClip.Right - rcClip.Left, 1, DSTINVERT
        PatBlt hDC, rcClip.Left, rcClip.Bottom, _
            1, -(rcClip.Bottom - rcClip.Top), DSTINVERT
        PatBlt hDC, rcClip.Right - 1, rcClip.Top, _
            1, rcClip.Bottom - rcClip.Top, DSTINVERT
        PatBlt hDC, rcClip.Right, rcClip.Bottom - 1, _
            -(rcClip.Right - rcClip.Left), 1, DSTINVERT
        
        sz = CStr(rcClip.Right - rcClip.Left) & "x" & _
                   CStr(rcClip.Bottom - rcClip.Top)
                   
        '''''''''''''''''''''''''''''''''''''''''''''''''''
        '' ... and center it in the rectangle
        GetTextExtentPoint32 hDC, sz, Len(sz), sExtent
        dx = sExtent.cx
        dy = sExtent.cy
        x = (rcClip.Right + rcClip.Left - dx) / 2
        y = (rcClip.Bottom + rcClip.Top - dy) / 2
        hdcBits = CreateCompatibleDC(hDC)
        SetTextColor hdcBits, &HFFFFFF
        SetBkColor hdcBits, &H0

        '''''''''''''''''''''''''''''''''''''''''''''''''
        '' Output the text to the DC
        hbm = CreateBitmap(dx, dy, 1, 1, 0&)
        If (hbm) Then
            hbm = SelectObject(hdcBits, hbm)

            '''''''''''''''''''''''''''''''''''''''''''''''''
            '' Modified Jul 11 1997; Brad Perkins
            '' ExtTextOut hdcBits, 0, 0, 0, 0, sz, Len(sz), 0
            '' ExtTextOut dosn't work in VB for some
            '' reason!!! So...
            
            '''''''''''''''''''''''''''''''''''''''''''''''''
            '' ... Use TextOut which seems ok
            TextOut hdcBits, 0, 0, sz, Len(sz)
            
            '''''''''''''''''''''''''''''''''''''''''''''''''
            '' Paste it in
            BitBlt hDC, x, y, dx, dy, hdcBits, 0, 0, SRCINVERT
            
            ''''''''''''''''''''''''''''''''''''''''''''''''''
            '' restore the origional handle; Clean up
            hbm = SelectObject(hdcBits, hbm)
            DeleteObject hbm
        
        End If '' hbm
        DeleteDC hdcBits
    
    ''''''''''''''''''''''''''''''''''''''''''''''''''''
    '' if Not IsRectEmpty(rcClip)
    End If

''''''''''''''''''''''''''''''''''''''''''''''''''''
'' End DrawSelect(...)
End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' FUNCTION:
''
'' NormalizeRect (RECT * Rct)
''
'' Purpose: If the rectangle coordinates are reversed, swaps
'' them. This is used to make sure that the first coordinate
'' of our rect is the upper left, and the second is lower
'' right.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
Public Sub NormalizeRect(ByRef Rct As RECT)
    If (Rct.Right < Rct.Left) Then Swap Rct.Right, Rct.Left
    If (Rct.Bottom < Rct.Top) Then Swap Rct.Bottom, Rct.Top
End Sub



'''''''''''''''''''''''''''''''''''''''''''''''''''''''
'' Hack to Replace C++ Template Function
'' Brad Perkins Oct 7 1997

Public Sub Swap(ByRef x As Long, ByRef y As Long)
    x = x Xor y
    y = x Xor y
    x = x Xor y
End Sub







